/** 
 * ----------------------------------------------------------------
 * $Id: libdyalogsqlite_c.c,v 1.1.1.1 2005/09/19 09:14:29 clerger Exp $
 * Copyright (C) 2003, 2004, 2005 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  \file  libdyalogsqlite_c.c 
 *  \brief Support for libdyalogsqlite.pl
 *
 * ----------------------------------------------------------------
 * Description
 * 
 * ----------------------------------------------------------------
 */

#include <stdio.h>
//#include <unistd.h>
#include <sqlite3.h>

#include "libdyalog.h"
#include "builtins.h"

sqlite3 *
DyALog_sqlite3_open(char *file) 
{
    int rc;
    sqlite3 *db;
    rc = sqlite3_open(file, &db);
    if( rc ){
        fprintf(stderr, "Can't open database: %s\n", sqlite3_errmsg(db));
        sqlite3_close(db);
        exit(1);
    }
    return db;
}

sqlite3_stmt *
DyALog_sqlite3_prepare(sqlite3 *db,const char *stmt) 
{
    sqlite3_stmt *pStmt;
    char *zTail;
    int rc;
    rc = sqlite3_prepare(db,stmt,-1,&pStmt,&zTail);
    if (rc) {
        fprintf(stderr, "Can't prepare stmt: %s\n",stmt);
        exit(1);
    }
    return pStmt;
}

Bool
DyALog_sqlite3_getvalue (sqlite3_stmt *pstmt,
                        int column,
                        sfol_t value
                        )
{
    int type = sqlite3_column_type(pstmt,column);

    switch ((int) type) {

        case SQLITE_INTEGER:
            return Unify( DFOLINT(sqlite3_column_int(pstmt,column)), Key0,
                          value->t,value->k);
            break;
            
        case SQLITE_FLOAT:
            return Unify( DFOLFLT(sqlite3_column_double(pstmt,column)), Key0,
                          value->t,value->k);
            break;
            
        case SQLITE_TEXT:
        case SQLITE_BLOB:
            return Unify( Create_Atom(strdup(sqlite3_column_text(pstmt,column))), Key0,
                          value->t,value->k);
            break;
            
        case SQLITE_NULL:                /* we return a null list */
            return Unify( FOLNIL, Key0,
                          value->t, value->k );
            break;
    }
}


Bool
DyALog_sqlite3_bindvalues( sqlite3_stmt *pstmt,
                           sfol_t values
                           ) 
{
    int n = sqlite3_bind_parameter_count(pstmt);
    int i;
    Sdecl(x);
    for(i=1; i <= n; i++ ) {
        SFOL_Deref(values);
        if (!FOLPAIRP(values->t))
            Fail;
        x=FOLPAIR_CAR(values->t);
        Sk(x)=values->k;
        values->t=FOLPAIR_CDR(values->t);
        Deref(x);
        if (FOLSMBP(x)) {
            sqlite3_bind_text(pstmt,i,FOLSMB_NAME(x),-1,SQLITE_STATIC);
        } else if (FOLINTP(x)) {
            sqlite3_bind_int(pstmt,i,(int)CFOLINT(x));
        } else if (FOLFLTP(x)) {
            sqlite3_bind_double(pstmt,i,(double)CFOLFLT(x));
        } else if (FOLNILP(x)) {
            sqlite3_bind_null(pstmt,i);
        } else {
            Fail;
        }
    }
    Succeed;
}
